//
//  CWDownloadQueue.m
//  SC68 Player
//
//  Created by Fredrik Olsson on 2008-10-30.
//  Copyright 2008 Jayway AB. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "CWDownloadQueue.h"


@implementation CWDownloadQueue

#pragma mark --- Properties

@synthesize delegate = _delegate;


#pragma mark --- Life cycle

-(id)initWithDelegate:(id<CWDownloadQueueDelegate>)delegate;
{
	self = [super init];
  if (self) {
  	self.delegate = delegate;
    _downloads = [[NSMutableArray alloc] init];
  }
  return self;
}

-(void)dealloc
{
	[_downloads release];
  [super dealloc];
}


#pragma mark --- Manage downloads

-(void)ensureDirectoryExistsForFile:(NSString*)path;
{
	path = [path stringByDeletingLastPathComponent];
  [[NSFileManager defaultManager] createDirectoryAtPath:path withIntermediateDirectories:YES attributes:nil error:NULL];
}

-(void)fireDelegate:(NSArray*)data;
{
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	NSArray* download = [data objectAtIndex:1];
  NSURL* URL = [download objectAtIndex:0];
  NSString* path = [download objectAtIndex:1];
  void* context = [[download objectAtIndex:2] pointerValue];
	switch ([[data objectAtIndex:0] intValue]) {
    case 0:
    	tempResult = [self.delegate downloadQueue:self shouldDownloadURL:URL toFile:path context:context];
			break;
    case 1:
    	[self.delegate downloadQueue:self didDownloadURL:URL toFile:path context:context];
      break;
    case 2:
      [self.delegate downloadQueue:self failedDownloadURL:URL toFile:path context:context error:[data objectAtIndex:2]];
    default:
      break;
  }
  [pool release];
}

-(void)performDownloadQueue;
{
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
  [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
	while ([_downloads count] > 0) {
		NSArray* download = [_downloads objectAtIndex:0];
    NSURL* URL = [download objectAtIndex:0];
    NSString* path = [download objectAtIndex:1];
    BOOL shouldDownload = YES;
		if ([self.delegate respondsToSelector:@selector(downloadQueue:willDownloadURL:toFile:context:)]) {
    	[self performSelectorOnMainThread:@selector(fireDelegate:) withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:0], download, nil] waitUntilDone:YES];
    }
    if (shouldDownload) {
      NSURLRequest* request = [NSURLRequest requestWithURL:URL];
      NSURLResponse* response = nil;
      NSError* error = nil;
      NSData* data = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
      if (data) {
      	[self ensureDirectoryExistsForFile:path];
        if ([data writeToFile:path options:NSAtomicWrite error:&error]) {
          if ([self.delegate respondsToSelector:@selector(downloadQueue:didDownloadURL:toFile:context:)]) {
			    	[self performSelectorOnMainThread:@selector(fireDelegate:) withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:1], download, nil] waitUntilDone:YES];
          }
        }
      }
      if (error != nil) {
        if ([self.delegate respondsToSelector:@selector(downloadQueue:failedDownloadURL:toFile:context:error:)]) {
		    	[self performSelectorOnMainThread:@selector(fireDelegate:) withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:2], download, error, nil] waitUntilDone:YES];
        }
      }
		}
    [_downloads removeObjectAtIndex:0];
    [pool drain];
  }
  _downloading = NO;
  [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
  [pool release];
}


-(BOOL)startDownloadQueue;
{
	@synchronized(self) {
    if (![self isDownloading]) {
   		_downloading = YES;
      [self performSelectorInBackground:@selector(performDownloadQueue) withObject:nil]; 
      return YES;
    }
	}
  return NO;
}

-(BOOL)isDownloading;
{
	return _downloading;
}

-(BOOL)downloadURL:(NSURL*)URL toFile:(NSString*)path context:(void*)context;
{
	NSArray* download = [NSArray arrayWithObjects:URL, path, [NSValue valueWithPointer:context], nil];
  if ([download count] != 3) {
  	[NSException raise:NSInvalidArgumentException format:@"URL and path must not be nil"];
  }
  [_downloads addObject:download];
  return [self startDownloadQueue];
}

@end
